<?php

namespace NCSU\Auth\Framework\Laravel;

use Closure;
use Illuminate\Support\Facades\Auth;
use NCSU\Auth\Adapter\ShibAuthAdapter;
use NCSU\Auth\AuthService;
use NCSU\Auth\Http\Request;

class ShibbolethAuthenticate
{
    /**
     * @var AuthService
     */
    private $authService;

    public function __construct()
    {
        $this->authService = new AuthService(new ShibAuthAdapter(Request::createFromGlobals()));
    }

    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure  $next
     * @return mixed
     */
    public function handle($request, Closure $next)
    {
        $response = $this->authService->authenticate();
        $identity = $response->getIdentity();

        $user = $this->findOrCreateUser($identity);
        Auth::login($user);

        return $next($request);
    }

    /**
     * Return user if exists; create and return if doesn't
     *
     * @param $shibbolethUser
     * @return User
     */
    private function findOrCreateUser($shibbolethUser)
    {
        $userModelClass = config('auth.providers.users.model');

        if ($authUser = $userModelClass::where('name', $shibbolethUser)->first()) {
            return $authUser;
        }

        return $userModelClass::create([
            'name' => $shibbolethUser,
            'email' => $shibbolethUser . '@ncsu.edu',
            'password' => str_random(64),
        ]);
    }
}
